"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deprecatedPlugin = void 0;
const graphql_1 = require("graphql");
const deprecatedPlugin = () => {
    return {
        requestDidStart: (initialRequestContext) => __awaiter(void 0, void 0, void 0, function* () {
            const newDeprecations = [];
            return {
                executionDidStart(executionRequestContext) {
                    return __awaiter(this, void 0, void 0, function* () {
                        return {
                            willResolveField({ source, args, contextValue, info }) {
                                const fieldDef = getFieldDef(info);
                                const directive = info.schema.getDirective('deprecated');
                                const fieldAstNode = fieldDef.astNode;
                                if (!directive || !fieldAstNode) {
                                    return;
                                }
                                const directiveValue = (0, graphql_1.getDirectiveValues)(directive, fieldAstNode);
                                if (!directiveValue) {
                                    return;
                                }
                                newDeprecations.push({
                                    field: info.fieldName,
                                    reason: directiveValue.reason,
                                });
                                return (_error, _result) => { };
                            },
                        };
                    });
                },
                willSendResponse(requestContext) {
                    return __awaiter(this, void 0, void 0, function* () {
                        if (newDeprecations.length > 0) {
                            if (!requestContext.response.extensions) {
                                requestContext.response.extensions = {};
                            }
                            if (!requestContext.response.extensions.deprecations) {
                                requestContext.response.extensions.deprecations = [];
                            }
                            requestContext.response.extensions.deprecations =
                                requestContext.response.extensions.deprecations.concat(newDeprecations);
                        }
                    });
                },
            };
        }),
    };
};
exports.deprecatedPlugin = deprecatedPlugin;
function getFieldDef(info) {
    const parentTypeFields = info.parentType.getFields();
    return parentTypeFields[info.fieldName];
}
